from django.db import models
from userprofile_app.models import Industry
from .utils import cover_image_upload_to
from crea_app.storages import UthoStorage

from django.contrib.auth import get_user_model

User = get_user_model()

# Create your models here.
class Forum(models.Model):
    PERMISSION_CHOICES = [
        ('public', 'Public'),
        ('private', 'Private'),
    ]

    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name="forums")
    title = models.CharField(max_length=255)
    categories = models.ManyToManyField(Industry, related_name="forums")
    short_description = models.TextField()
    hashtags = models.JSONField(default=list)
    permission_to_join = models.CharField(max_length=10, choices=PERMISSION_CHOICES)
    invite_user_ids = models.JSONField(default=list)
    created_at = models.DateTimeField(auto_now_add=True)
    is_deleted = models.BooleanField(default=False)

    cover_image = models.FileField(
        storage=UthoStorage(),
        upload_to=cover_image_upload_to,
        blank=True,
        null=True
    )

    def __str__(self):
        return f'{self.title}_{self.id}'

class ForumMembership(models.Model):
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name="forum_memberships")
    forum = models.ForeignKey(Forum, on_delete=models.CASCADE, related_name="memberships")
    is_active = models.BooleanField(default=True)  # For managing exits
    joined_at = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return f'{self.forum.title}_{self.user.id}_{self.forum.id}'

class ForumInvitation(models.Model):
    invited_by = models.ForeignKey(User, on_delete=models.CASCADE, related_name="sent_invitations")
    forum = models.ForeignKey(Forum, on_delete=models.CASCADE, related_name="invitations")
    invited_user = models.ForeignKey(User, on_delete=models.CASCADE, related_name="received_invitations")
    status = models.CharField(max_length=10, choices=[('pending', 'Pending'), ('accepted', 'Accepted'), ('declined', 'Declined')], default='pending')
    invited_at = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return f'{self.invited_user.mobile_or_email}_{self.invited_user.id}_{self.forum.id}'
    
class ForumQuestion(models.Model):
    forum = models.ForeignKey(Forum, on_delete=models.CASCADE, related_name="questions")
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name="questions")
    question = models.CharField(max_length=255)
    content = models.TextField()
    tags = models.JSONField(default=list)
    created_at = models.DateTimeField(auto_now_add=True)
    is_deleted = models.BooleanField(default=False)

    def __str__(self):
        return f"Question: {self.question} (Forum: {self.forum.title})"

class Comment(models.Model):
    question = models.ForeignKey(ForumQuestion, on_delete=models.CASCADE, related_name='question_comments')
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name='question_comments')
    comment = models.TextField()
    created_at = models.DateTimeField(auto_now_add=True)
    is_deleted = models.BooleanField(default=False)

class Like(models.Model):
    question = models.ForeignKey(ForumQuestion, on_delete=models.CASCADE, related_name='question_likes')
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name='question_likes')
    comment = models.TextField()
    created_at = models.DateTimeField(auto_now_add=True)
    is_deleted = models.BooleanField(default=False)

class Notification(models.Model):
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name='forum_notifications')
    message = models.CharField(max_length=255)
    is_read = models.BooleanField(default=False)
    created_at = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return f"Notification for {self.user.username}"